local lua_CheckPointTime= nw.createParser("CheckPointTime", "Checks Difference between CheckPoint Log Time and when it was processed on LogDecoder")


--[[
    
    DESCRIPTION
		See https://community.rsa.com/message/872046#comment-872046
        Takes a log message from a CheckPoint:
		
		May 16 2016 15:24:26: %CHKPNT-1-030050: drop,192.168.202.243,inbound,eth0,192.168.202.243,23105,192.168.6.6,22,ssh,tcp, , , , , , , , , , ,alert, ,Local interface address spoofing, , , , , , , , , , , ,16May2016 15:24:26,0,VPN-1 & FireWall-1, , , , , , , , , , , , , , , , , , , , , , , , , , , , , , , ,030050, , , , , , , , , , , , , , , , , , , , ,
		
		
    AUTHOR
    
    	david.waugh2@rsa.com 


    VERSION
		1.0 Inital Parser Request
		2.0 Update for Timestamp feed
	
    DEPENDENCIES
		Make sure that header 2 in the checkpoint parser is replaced as follows:
	 
			<HEADER
                id1="0002"
                id2="0002"
                devts="MDYTS(hmonth,hday,hyear,htime)"
                content="&lt;hmonth&gt; &lt;hday&gt; &lt;hyear&gt; &lt;htime&gt; %CHKPNT-&lt;hlevel&gt;-&lt;messageid&gt;: &lt;!payload&gt;" />

				
		Make Sure that in your /etc/netwitness/ng/envision/etc/table-map-custom.xml that the following keys are set to either None or Transient
        
		<!- BEGIN Check Point Date Time Header Fields -->
			<mapping envisionName="hyear" nwName="year" flags="None" format="Text"/>
			<mapping envisionName="hmonth" nwName="month" flags="None" format="Text"/>
			<mapping envisionName="hday" nwName="day" flags="None" format="Text"/>
			<mapping envisionName="htime" nwName="htime" flags="None" format="Text"/>
			<mapping envisionName="fld49" nwName="fld49" flags="None" format="Text" />
			<mapping envisionName="fld49" nwName="fld49" flags="None" format="Text" />
			
		<!- END Check Point Date Time Header Fields -->

	Note fld49 normal holds the log timestamp
	
	   Add the following to your index-concentrator-custom.xml files.
	   
		<key description="Seconds Delay"  level="IndexValues" name="seconds.delay" format="UInt32" defaultAction="Open" valueMax="10000" />
		<key description="Event Time String" level="IndexValues" name="event.time.str" format="Text" valueMax="2500000"/>

--]]
function days_from_epoch(y, m, d)
-- This algorithm came from
-- http://howardhinnant.github.io/date_algorithms.html
--
    if m <= 2 then
      y = y - 1
      m = m + 9
    else
      m = m - 3
    end
    local era = math.floor(y/400)
    local yoe = y - era * 400                                           -- [0, 399]
    local doy = math.modf((153*m + 2)/5) + d-1                          -- [0, 365]
    local doe = yoe * 365 + math.modf(yoe/4) - math.modf(yoe/100) + doy -- [0, 146096]
    return era * 146097 + doe - 719468
end

function epochtime(year,month,day,hour,minute,second)
  local days=days_from_epoch(year, month, day)
  return (days*86400 + hour*60*60 +minute*60 + second)
end

---local eventdesc ='Server patching. RFC 123456 [From 15/02/2016 08:00:00 to 15/02/2016 17:00:00 (GMT+1.00) multiple operations]'
---local hdatetime ='2016-02-15T11:17:35Z'
-- Write meta into the following meta key(s)
lua_CheckPointTime:setKeys({
	nwlanguagekey.create("alert",nwtypes.Text),
	nwlanguagekey.create("seconds.delay",nwtypes.UInt32)
})

--[[  Converting a three letter month string into a month
number ... sounds easy enough (and it is!) but there are a
lot of alternative ways of doing it ...
]]

function month2digit(month)
        month = month:upper()
        monthNum=0
        if month == 'JAN' then
                monthNum=1
        elseif month == 'FEB' then
                monthNum=2
        elseif month == 'MAR' then
                monthNum=3
        elseif month == 'APR' then
                monthNum=4
        elseif month == 'MAY' then
                monthNum=5
        elseif month == 'JUN' then
                monthNum=6
        elseif month == 'JUL' then
                monthNum=7
        elseif month == 'AUG' then
                monthNum=8
        elseif month == 'SEP' then
                monthNum=9
        elseif month == 'OCT' then
                monthNum=10
        elseif month == 'NOV' then
                monthNum=11
        elseif month == 'DEC' then
                monthNum=12
        end
        return monthNum
end
	
function lua_CheckPointTime:sessionBegin()
    HeaderTime = nil
	EventTime = nil
	HeaderMonth=nil
	HeaderYear=nil
	HeaderDay=nil
	devicetype = nil
	msgid=nil
	EventTimeStr=nil
end

function lua_CheckPointTime:CheckDevice(index,dtype)
	if dtype == 'checkpointfw1' then
		devicetype = dtype
	end
	self:register()
end

function lua_CheckPointTime:CheckMSGID(index,mymsgid)
	-- Define CheckPoint Messages we wish to track.
	if mymsgid=="060020:21" then
	  msgid="060020:21"
	elseif mymsgid=="050100:25" then
	  msgid="050100:25"
	elseif mymsgid=="030050:23" then
	  msgid="030050:23"
	else
	  msgid=nil
	end
	self:register()
end

function lua_CheckPointTime:getEventTime(index,myEventTime)
		EventTime=myEventTime
		----nw.logInfo("EventTime " .. EventTime)
		self:register()
	
end

function lua_CheckPointTime:getHeaderYear(index,myHeaderYear)
		HeaderYear=myHeaderYear
		----nw.logInfo("HeaderYear: " .. HeaderYear)
	
	self:register()
end

function lua_CheckPointTime:getEventTimeStr(index,myEventTimeStr)
		EventTimeStr=myEventTimeStr
			
	self:register()
end
function lua_CheckPointTime:getHeaderMonth(index,myHeaderMonth)
		HeaderMonth=myHeaderMonth
		----nw.logInfo("HeaderMonth: " .. HeaderMonth)
	
	self:register()
end

function lua_CheckPointTime:getHeaderDay(index,myHeaderDay)
		HeaderDay=myHeaderDay
		----nw.logInfo("HeaderDay: " .. HeaderDay)
		self:register()
end


function lua_CheckPointTime:getHeaderTime(index,myHeaderTime)
		HeaderTime=myHeaderTime
		----nw.logInfo("HeaderTime: " .. HeaderTime)
		self:register()
end

function lua_CheckPointTime:register()

    if devicetype then
				
		if (EventTime) and HeaderTime and HeaderYear and HeaderMonth and HeaderDay and msgid and EventTimeStr then
			--nw.logInfo("Meta Registered")
			--nw.logInfo("EventTime: " .. EventTime)
			--nw.logInfo("HeaderTime: " .. HeaderTime)
			--nw.logInfo("HeaderYear: " .. HeaderYear)
			--nw.logInfo("HeaderMonth: " .. HeaderMonth)
			--nw.logInfo("HeaderDay: " .. HeaderDay)
			--nw.logInfo("msg.id: " .. msgid)
			--nw.logInfo("EventTimeStr: " .. EventTimeStr)
			
			--[[  Example REsults
			May 16 20:49:55 LOGDECCOL1 NwLogDecoder[24811]: [Lua] [info] Meta Registered
			May 16 20:49:55 LOGDECCOL1 NwLogDecoder[24811]: [Lua] [info] EventTime: 16May2016 20:47:52
			May 16 20:49:55 LOGDECCOL1 NwLogDecoder[24811]: [Lua] [info] HeaderTime: 20:47:52:
			May 16 20:49:55 LOGDECCOL1 NwLogDecoder[24811]: [Lua] [info] HeaderYear: 2016
			May 16 20:49:55 LOGDECCOL1 NwLogDecoder[24811]: [Lua] [info] HeaderMonth: May
			May 16 20:49:55 LOGDECCOL1 NwLogDecoder[24811]: [Lua] [info] HeaderDay: 16
			--]]

			---Do All The Work in Here
			local Day,Month,Year,Space,Hour,Minute,Second=string.match(EventTime,"(%d+)(%a+)(%d+)(%s*)(%d+):(%d+):(%d+)")
			--nw.logInfo("Day: " .. Day)
			--nw.logInfo("Month: " .. Month)
			--nw.logInfo("Year: " .. Year)
			--nw.logInfo("Hour: " .. Hour)
			--nw.logInfo("Minute: " .. Minute)
			--nw.logInfo("Second: " .. Second)
			
			local EventEpochTime=epochtime(tonumber(Year),tonumber(month2digit(Month)),tonumber(Day),tonumber(Hour),tonumber(Minute),tonumber(Second))
			--nw.logInfo("EventEpochTime " .. EventEpochTime)
		
			local Hour,Minute,Second= string.match(HeaderTime,"(%d+):(%d+):(%d+):")
			local HeaderEpochTime =epochtime(tonumber(HeaderYear),tonumber(month2digit(HeaderMonth)),tonumber(HeaderDay),tonumber(Hour),tonumber(Minute),tonumber(Second))
			--nw.logInfo("HeaderEpochTime " .. HeaderEpochTime)
			
			local WeekDay,Month,Day,Hour,Minute,Second,Year=string.match(EventTimeStr,"(%a+) (%a+) (%d+) (%d+):(%d+):(%d+) UTC (%d+)")
			--nw.logInfo("WeekDay: " .. WeekDay)
			--nw.logInfo("Month: " .. Month)
			--nw.logInfo("Day: " .. Day)
			--nw.logInfo("Year: " .. Year)
			--nw.logInfo("Hour: " .. Hour)
			--nw.logInfo("Minute: " .. Minute)
			--nw.logInfo("Second: " .. Second)
		
		    local EventTimeStrEpochTime=epochtime(tonumber(Year),tonumber(month2digit(Month)),tonumber(Day),tonumber(Hour),tonumber(Minute),tonumber(Second))
			--nw.logInfo("EventTimeStrEpochTime " .. EventTimeStrEpochTime)
			
			local collectiondelay=0
			local decoderdelay=0
				if HeaderEpochTime > EventEpochTime then
				-- Compare Header Time with Time in Log Field 
				    local collectiondelay = HeaderEpochTime - EventEpochTime
					--- Define Warning as 120 seconds delayed
					local warning=120
					--- Define Critical as 180 seconds delayed
					local critical=180
					
					local level=nil;
					if collectiondelay >= warning and collectiondelay < critical then
					  level="WARNING"
					elseif collectiondelay > critical then
					  level="CRITICAL"
					end
					
					if level then
						--nw.logInfo(level .. " CheckPoint Log Collection Delay" )
						nw.createMeta(self.keys["alert"], level .. " CheckPoint Log Collection Delay")
					end
					
				end
				
				if EventTimeStrEpochTime> HeaderEpochTime then
				-- Compare Header Time and Time of our TimeStamp to check for delay in processing.
				    local decoderdelay = EventTimeStrEpochTime - HeaderEpochTime
					--- Define Warning as 120 seconds delayed
					local warning=120
					--- Define Critical as 180 seconds delayed
					local critical=180
					
					local level=nil;
					if decoderdelay >= warning and decoderdelay < critical then
					  level="WARNING"
					elseif decoderdelay > critical then
					  level="CRITICAL"
					end
					
					if level then
						--nw.logInfo(level .. " CheckPoint Log Decoder Delay" )
						nw.createMeta(self.keys["alert"], level .. " CheckPoint Log Decoder Delay")
					end
					
				end
			
			nw.createMeta(self.keys["seconds.delay"],math.max(collectiondelay,decoderdelay))
		end
	end	
end


-- declare what tokens and events we want to match
lua_CheckPointTime:setCallbacks({
    [nwevents.OnSessionBegin] = lua_CheckPointTime.sessionBegin,
    [nwlanguagekey.create("device.type", nwtypes.Text)] = lua_CheckPointTime.CheckDevice, 
    [nwlanguagekey.create("fld49", nwtypes.Text)] = lua_CheckPointTime.getEventTime, 
	[nwlanguagekey.create("hyear", nwtypes.Text)] = lua_CheckPointTime.getHeaderYear, 
	[nwlanguagekey.create("hmonth", nwtypes.Text)] = lua_CheckPointTime.getHeaderMonth,
	[nwlanguagekey.create("hday", nwtypes.Text)] = lua_CheckPointTime.getHeaderDay,
    [nwlanguagekey.create("htime", nwtypes.Text)] = lua_CheckPointTime.getHeaderTime, 
    [nwlanguagekey.create("msg.id", nwtypes.Text)] = lua_CheckPointTime.CheckMSGID,
	[nwlanguagekey.create("event.time.str", nwtypes.Text)] = lua_CheckPointTime.getEventTimeStr, 	
})
